//
// This file is part of Nolimips, a MIPS simulator with unlimited registers
// Copyright (C) 2003, 2004, 2006 Benoit Perrot <benoit@lrde.epita.fr>
//
// Nolimips is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Nolimips is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Nolimips.  If not, see <http://www.gnu.org/licenses/>.
//

#include "vm/virtual_machine.hh"

#include "misc/has.hh"

#include "inst/program.hh"

namespace vm
{

  void
  VirtualMachine::reset()
  {
    memory_.reset();
    mmu_.reset();
    cp0_.reset();
    cpu_.reset();
  }

  void
  VirtualMachine::load_program(const inst::Program &program)
  {
    reset();

    misc::unique_string main("main");
    if (! program.text_section ().has_label(main))
      {
        std::cerr << "No `main' label in assembly file." << std::endl;
        exit_set(exit_runtime);
        return;
      }
    main_offset_ = program.text_section().get_offset(main);
    mmu_.data_store(program.data_section());
    mmu_.inst_store(program.text_section());
  }

  void
  VirtualMachine::execute()
  {
    // FIXME: VirtualMachine::reset()?
    cp0_.reset();
    cpu_.reset();

    // FIXME: precondition on loaded program
    cpu_.set_pc(main_offset_);
    while (!cp0_.fatal_exception())
      {
        cpu_.step();
        cp0_.set_count(cp0_.get_count() + 1);
      }

    // FIXME: this is not the good test, but for the moment it suffices
    if (cp0_.get_register(Cp0::cause) != Cp0::interrupt)
      exit_set(exit_runtime);
  }

  void
  VirtualMachine::execute(bool trace_p)
  {
    // FIXME: VirtualMachine::reset()?
    cp0_.reset();
    cpu_.reset();

    if (status_ == stop)
      return;

    cpu_.set_pc(main_offset_);
    cpu_.set_trace(trace_p);
    while (status_ == run)
      {
        cpu_.step();
        cp0_.set_count(cp0_.get_count() + 1);
        if (cp0_.fatal_exception())
          {
            status_ = halt;
            std::cerr << "Program exited." << std::endl;
            break;
          }
        if (has(breakpoints_, cpu_.get_pc()))
          {
            std::cout << "Breakpoint at pc = " << cpu_.get_pc() << std::endl;
            status_ = pause;
          }
        if (mode_ == step)
          status_ = pause;
      }
  }

  void
  VirtualMachine::add_breakpoint(int offset)
  {
    assertion(0 < offset /* && (offset / 4) <= text_section.size() */);
    breakpoints_.push_back(offset + 4);
  }


} // namespace vm
